import datetime
import typing
from moonlistclient.models.discord import DiscordUser, DiscordGuild
from pydantic import BaseModel


class BotStat(BaseModel):
    id: int
    time: datetime.datetime
    bot_id: int
    users: int
    guilds: int
    shards: int


class BotOptions(BaseModel):
    github: typing.Optional[str]
    website: typing.Optional[str]
    documentation: typing.Optional[str]
    support_server: typing.Optional[str]


class BaseBot(BaseModel):
    id: int
    currently: int
    added_at: datetime.datetime
    short_description: str
    description: str
    invite: str
    prefix: str
    tags: list
    lib: str
    hidden: bool
    pinned: bool
    options: BotOptions
    badges: list
    verificated: bool
    stat: typing.Optional[BotStat]
    count_bumps: int
    discord: DiscordUser
    owner: DiscordUser
    owners: typing.List[typing.Optional[DiscordUser]]


class RawBot(BaseBot):
    bot_id: int
    count_bumps: typing.Optional[int]
    discord: typing.Optional[DiscordUser]
    owner: int
    owners: typing.List[typing.Optional[int]]


class BotInEdit(BaseModel):
    short_description: typing.Optional[str]
    description: typing.Optional[str]
    invite: typing.Optional[str]
    prefix: typing.Optional[str]
    tags: typing.Optional[typing.List[str]]
    lib: typing.Optional[str]
    hidden: typing.Optional[bool]
    owners: typing.Optional[typing.List[int]]
    options: typing.Optional[BotOptions]
    badges: typing.Optional[typing.List[str]]
    pinned: typing.Optional[bool]
    verificated: typing.Optional[bool]


class BaseComment(BaseModel):
    id: int
    user_id: int
    mark: int
    created_at: datetime.datetime
    edited_at: typing.Optional[datetime.datetime]
    message: str


class BaseServer(BaseModel):
    id: int
    owner_id: int
    added_at: datetime.datetime
    invite: str
    short_description: str
    description: str
    features: dict
    tags: typing.List[str]
    discord: DiscordGuild


class ServerInEdit(BaseModel):
    invite: typing.Optional[str]
    short_description: typing.Optional[str]
    description: typing.Optional[str]
    tags: typing.Optional[typing.List[str]]


import aiohttp
import asyncio
import logging
import typing
import json
from moonlistclient.exceptions import *


logger = logging.getLogger(__name__)


class ApiClient:
    def __init__(self, api_key: str, **kwargs):
        self.loop = kwargs.get("loop") or asyncio.get_event_loop()
        self.session = kwargs.get("session") or aiohttp.ClientSession(loop=self.loop)
        self.base_url: str = ""
        self.api_key: str = api_key

        if not self.api_key:
            raise Unauthorized("You didn't provide an api key")

    async def _json_or_text(self, response: aiohttp.ClientResponse) -> typing.Union[list, dict, str]:
        text = await response.text()
        if 'application/json' in response.headers.get('Content-Type', 'text/plain'):
            return json.loads(text)
        return text

    async def request(self, method: str, endpoint: str, **kwargs):
        if "headers" not in kwargs.keys():
            kwargs["headers"] = {
                "Authorization": self.api_key
            }

        logger.info(f"Make request for {endpoint} with method {method}")

        async with self.session.request(method, self.base_url+endpoint, **kwargs) as response:
            response_json = await self._json_or_text(response)
            logger.info(f"Response of request to {endpoint} with method {method}: {response_json}")
            logger.info(f"Response status of request to {endpoint} with method {method}: {response.status}")

            if 300 > response.status >= 200:
                return response_json

            if response.status == 404:
                raise NotFound(response_json)
            elif response.status == 400:
                raise BadRequest(response_json)
            elif response.status == 403:
                raise Forbidden(response_json)
            elif response.status == 429:
                raise RateLimited(response_json)
            else:
                raise ServerError(response_json)

    async def get_bot(self, bot_id: int) -> dict:
        return await self.request("GET", f"/bots/{bot_id}")

    async def get_bots(self, limit: int = 20, skip: int = 0) -> list:
        return await self.request(
            "GET",
            f"/bots",
            params={"limit": limit, "skip": skip}
        )

    async def get_bumps(self, bot_id: int) -> list:
        return await self.request("GET", f"/bots/{bot_id}/bumps")

    async def get_bumps_count(self, bot_id: int) -> dict:
        return await self.request("GET", f"/bots/{bot_id}/bumps/count")

    async def get_comments(self, bot_id: int) -> list:
        return await self.request("GET", f"/comments/{bot_id}")

    async def get_user_reports(self, user_id: int) -> list:
        return await self.request("GET", f"/reports/{user_id}")

    async def get_reports(self) -> list:
        return await self.request("GET", f"/reports")

    async def get_user(self, user_id: int) -> dict:
        return await self.request("GET", f"/users/{user_id}")

    async def get_user_bots(self, user_id: int) -> list:
        return await self.request("GET", f"/users/{user_id}/bots")

    async def get_bot_stats(self, bot_id: int) -> list:
        return await self.request("GET", f"/bots/{bot_id}/stats")

    async def get_bot_stat(self, bot_id: int) -> dict:
        return await self.request("GET", f"/bots/{bot_id}/stat")

    async def post_bot_stat(self, bot_id: int, guilds: int, users: int, shards: int = 0):
        return await self.request(
            "POST",
            f"/bots/{bot_id}/stats",
            json={
                "guilds": guilds,
                "users": users,
                "shards": shards
            }
        )

    async def get_servers(self) -> list:
        return await self.request("GET", "/servers")

    async def get_server(self, guild_id: int) -> dict:
        return await self.request("GET", f"/servers/{guild_id}")

    async def close(self):
        await self.session.close()


import datetime
import typing

from pydantic import BaseModel
from moonlistclient.models.bases import BaseBot, RawBot, BaseComment, BaseServer


class Bot(BaseBot):
    bot_id: int


class Bump(BaseModel):
    id: int
    bot_id: int
    user_id: int
    time: datetime.datetime


class BumpsCount(BaseModel):
    count: int


class Comment(BaseComment):
    entity_id: int


class Report(BaseModel):
    id: int
    author_id: int
    entity_id: int
    created_at: datetime.datetime
    reason: str
    type: str
    state: int


class UserSocial(BaseModel):
    github: typing.Optional[str]
    website: typing.Optional[str]
    youtube: typing.Optional[str]
    discord: typing.Optional[str]
    twitter: typing.Optional[str]
    facebook: typing.Optional[str]
    vk: typing.Optional[str]


class User(BaseModel):
    id: int
    user_id: int
    access_level: int
    verificated: bool
    registered_at: datetime.datetime
    bio: typing.Optional[str]
    status: typing.Optional[str]
    bots: typing.List[RawBot] = []
    social: UserSocial
    badges: typing.List[str]


class Server(BaseServer):
    guild_id: int


import typing
from pydantic import BaseModel
from moonlistclient.models.bases import RawBot, BotInEdit, BaseComment, BaseServer, ServerInEdit


class BaseWebhookModel(BaseModel):
    trigger: str
    type: str
    user_id: int
    entity_id: int


class NewBotWebhook(BaseWebhookModel):
    bot: RawBot


class BotDeleteWebhook(BaseWebhookModel):
    reason: str
    bot: RawBot


class BotEditWebhook(BaseWebhookModel):
    new_data: BotInEdit


class BotEditCurrentlyWebhook(BaseWebhookModel):
    reason: typing.Optional[str]
    currently: int


class NewBumpWebhook(BaseWebhookModel):
    count_bumps: int


class NewCommentWebhook(BaseWebhookModel):
    comment: BaseComment


class NewServerWebhook(BaseWebhookModel):
    server: BaseServer


class ServerDeleteWebhook(BaseWebhookModel):
    reason: str
    server: BaseServer


class ServerEditWebhook(BaseWebhookModel):
    new_data: ServerInEdit


import typing
import discord
import logging

from .models import webhooks
from .exceptions import *
from aiohttp import web


TRIGGER_TO_MODEl = {
    "new_bot": webhooks.NewBotWebhook,
    "new_comment": webhooks.NewCommentWebhook,
    "bot_edit_currently": webhooks.BotEditCurrentlyWebhook,
    "bot_edit": webhooks.BotEditWebhook,
    "bot_delete": webhooks.BotDeleteWebhook,
    "new_bump": webhooks.NewBumpWebhook,
    "new_server": webhooks.NewServerWebhook,
    "server_edit": webhooks.ServerEditWebhook,
    "server_delete": webhooks.ServerDeleteWebhook
}
logger = logging.getLogger(__name__)


class Webhook:
    id: int

    def __init__(self, token: str, endpoint: str, trigger: str):
        self.token: str = token
        self.endpoint: str = endpoint
        self.trigger: str = trigger.lower()

        if self.trigger not in (
            "new_bot",
            "new_comment",
            "bot_edit_currently",
            "bot_edit",
            "bot_delete",
            "new_bump",
            "new_server",
            "server_edit",
            "server_delete"
        ):
            raise ClientException("An invalid webhook trigger was provided")

    def __getitem__(self, item):
        return self.__getattribute__(item)


class WebhookManager:
    _webserver: web.TCPSite

    def __init__(self, bot: discord.Client, port: int, host: str):
        self.bot: discord.Client = bot
        self.webhooks: typing.List[Webhook] = []
        self.port: int = port
        self.host = host
        self._last_id: int = 0
        self._app: web.Application = web.Application()
        self._is_ran: bool = False

    def add(self, webhook: Webhook) -> Webhook:
        webhook.id = self._generate_id()
        self.webhooks.append(webhook)
        return webhook

    def remove(self, webhook_id: int) -> typing.Optional[Webhook]:
        for webhook in self.webhooks:
            if webhook.id == webhook_id:
                self.webhooks.remove(webhook)
                return webhook
        return None

    def get_webhooks(self) -> typing.List[Webhook]:
        return self.webhooks

    def find(self, **kwargs):
        for webhook in self.webhooks:
            if all([webhook[key] == value for key, value in kwargs.items()]):
                return webhook

        return None

    def find_all(self, **kwargs):
        return [
            webhook
            for webhook in self.webhooks
            if all([webhook[key] == value for key, value in kwargs.items()])
        ]

    def _generate_id(self):
        new_id = self._last_id + 1
        self._last_id = new_id
        return new_id

    async def _handler(self, request: web.Request):
        data = await request.json()
        webhook = self.find(
            endpoint=str(request.rel_url),
            trigger=data.get("trigger")
        )
        if webhook is None:
            return web.Response(status=404, text="Not found")

        if request.headers.get("Authorization") != webhook.token:
            return web.Response(status=400, text="An invalid token was provided")

        self.bot.dispatch(data.get("trigger"), TRIGGER_TO_MODEl[data.get("trigger")](**data))
        return web.Response(status=204)

    async def run(self):
        if not self._is_ran:
            self._is_ran = True
            for webhook in self.webhooks:
                self._app.router.add_post(webhook.endpoint, self._handler)

            runner = web.AppRunner(self._app)
            await runner.setup()
            self._webserver = web.TCPSite(runner, self.host, self.port)
            await self._webserver.start()
            logger.info("Webserver is ran")

    async def close(self):
        if not self._is_ran:
            return

        await self._webserver.stop()
        self._is_ran = False


import typing
from pydantic import BaseModel


class DiscordEmoji(BaseModel):
    name: str
    id: str
    animated: bool


class DiscordUser(BaseModel):
    id: str
    username: str
    avatar: typing.Optional[str]
    discriminator: str
    public_flags: int
    bot: bool = False


class DiscordGuild(BaseModel):
    id: str
    name: str
    icon: typing.Optional[str]
    approximate_member_count: int
    approximate_presence_count: int
    region: str
    banner: typing.Optional[str]
    emojis: typing.List[DiscordEmoji]


from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='moonlistclient',
    version='0.6.8',
    description='Wrapper for the moonlist api',
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['moonlistclient', 'moonlistclient.models'],
    author_email='zyzel19@gmail.com',
    url="https://github.com/VadyChel/MoonlistClient",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "pydantic",
        "discord.py",
        "aiohttp"
    ]
)


from .client import MoonlistClient
from .exceptions import *
from .http import ApiClient
from .webhook_manager import *
from . import models
from .models.api import *
from .models.webhooks import *
from .models.bases import *